$set 10 #Windowmenu

$ #SendTo
# Gönder ...
$ #Shade
# Topla
$ #Iconify
# Ikonalaştır
$ #Maximize
# Azamileştir
$ #Raise
# Alçalt
$ #Lower
# Yükselt
$ #Stick
# Yapışık
$ #KillClient
# Öldür
$ #Close
# Kapat
