$set 16 #bsetroot

$ #MustSpecify
# %s : hata : -solid, -mod yada -gradient'den birisini belirlemek zorundasın\n
$ #Usage
# %s 2.0 : Tel'if hakkı (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <metin>         ekran belirlemesi\n\
  -mod <x> <y>             bölüşüm işlemi\n\
  -foreground, -fg <renk>  bölüşüm önalanı\n\
  -background, -bg <renk>  bölüşüm ardalanı\n\n\
  -gradient <kaplam>       geçişim kaplamı\n\
  -from <renk>             geçişim başlama rengi\n\
  -to <renk>               geçişim bitiş rengi\n\n\
  -solid <renk>            tek renk\n\n\
  -help                    bu yardım iletisini göster ve çık\n

