$set 14 #main

$ #RCRequiresArg
# hata : '-rc' bir argüman bekler\n
$ #DISPLAYRequiresArg
# hata : '-display' bir argüman bekler\n
$ #WarnDisplaySet
# ikaz : 'DISPLAY' verisini oturtamadım\n
$ #Usage
# Blackbox %s : (c) 2001 - 2002 Sean 'Shaleh' Perry\n\
  \t\t\t 1997 - 2000, 2002 Brad Hughes\n\n\
  -display <metin>\t\tekranı kullan.\n\
  -rc <metin>\t\t\tbaşka bir ayarlama dosyasını kullan.\n\
  -version\t\t\tnesil bilgisini gösterir ve çıkar.\n\
  -help\t\t\t\tbu yardım iletisini gösterir ve çıkar.\n\n
$ #CompileOptions
# Denetleme seçenekleri :\n\
  Bilgilendirme\t\t\t%s\n\
  Gölgeleme:\t\t\t%s\n\
  R8b'e göre tızla:\t%s\n\n
