$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen: viga X serveri küsitlemisel.\n  \
teine akahaldur juba töötab displeil %s.\n
$ #ManagingScreen
# BScreen::BScreen: haldame displeid %d reolutsioonis 0x%lx, vörvisügavusel %d\n
$ #FontLoadFail
# BScreen::LoadStyle(): ei saanud laadida fonti '%s'\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): ei saanud laadida vaikimisi määratud fonti.\n
$ #EmptyMenuFile
# %s: tühi menüüfail\n
$ #xterm
# xterm
$ #Restart
# Restart
$ #Exit
# Välju
$ #EXECError
# BScreen::parseMenuFile: [exec] viga, menüü pealkirja ja/või käsku pole määratud\n
$ #EXITError
# BScreen::parseMenuFile: [exit] viga, menüü pealkirja pole määratud\n
$ #STYLEError
# BScreen::parseMenuFile: [style] viga, menüü pealkirja ja/või failinime \
pole määratud\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] viga, menüü pealkirja pole määratud\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] viga, failinime pole määratud\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] viga, '%s' ei ole tavaline fail\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] viga, menüü pealkirja pole määratud\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] viga, menüü pealkirja pole määratud\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] viga, menüü pealkirja pole määratud\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] viga, kataloogi pole määratud\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] viga, '%s' pole kataloog\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] viga, '%s' pole olemas\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] viga, menüü pealkirja pole määratud\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# W: %4d x H: %4d

