$set 15 #Common

$ #Yes
# Evet
$ #No
# Hayır

$ #DirectionTitle
# Yön
$ #DirectionHoriz
# Ufki
$ #DirectionVert
# Dikey

$ #AlwaysOnTop
# Her zaman üstte

$ #PlacementTitle
# Yerleşim
$ #PlacementTopLeft
# Sol üst
$ #PlacementCenterLeft
# Sol orta 
$ #PlacementBottomLeft
# Sol alt
$ #PlacementTopCenter
# Üst orta
$ #PlacementBottomCenter
# Alt orta
$ #PlacementTopRight
# Sağ üst
$ #PlacementCenterRight
# Sağ orta
$ #PlacementBottomRight
# Sağ üst
