$set 1 #BaseDisplay

$ #XError
# %s : X hatası : %s ( %d ) opkodlar %d/%d\n  kaynak 0x%lx\n
$ #SignalCaught
# %s : %d sinyali alındı \n
$ #ShuttingDown
# kapatılıyorum\n
$ #Aborting
# kapatılıyorum ... çöküntüyü bırakıyorum\n
$ #XConnectFail
# BaseDisplay::BaseDisplay : X sunucusuna bağlanılınamadı .\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay : çalıştırırken kapatmada ekrana bağlanılınamadı \n
$ #BadWindowRemove
# BaseDisplay::eventLoop() : eylem kuyruğundaki 'kötü' pencereyi siliyorum \n
