$set 1 #BaseDisplay

$ #XError
# %s:  X'i viga: %s(%d) opkood %d/%d\n  resurss 0x%lx\n
$ #SignalCaught
# %s: signaal %d püütud\n
$ #ShuttingDown
# lõpetame\n
$ #Aborting
# katkestame... mälupilt tehtud\n
$ #XConnectFail
# BaseDisplay::BaseDisplay: ühenduse saamine X serveriga ebaõnnestus.\n
$ #CloseOnExecFail
# BaseDisplay::BaseDisplay: ühenduse märgistamine 'close-on-exec' ebaõnnestus\n
$ #BadWindowRemove
# BaseDisplay::eventLoop(): eemaldame vigase akna tominigute reast\n
