$set 14 #main

$ #RCRequiresArg
# viga: '-rc' vajab argumenti\n
$ #DISPLAYRequiresArg
# viga: '-display' vajab argumenti\n
$ #WarnDisplaySet
# hoiatus: ei saanud määrata keskonna muutujat 'DISPLAY'\n
$ #Usage
# Blackbox %s : (c) 2001 - 2002 Sean 'Shaleh' Perry\n\
  \t\t\t 1997 - 2000, 2002 Brad Hughes\n\n\
  -display <string>\t\tkasuta displei ühendust.\n\
  -rc <string>\t\t\tkasuta alternatiivseid resursse.\n\
  -version\t\t\tnäita versiooninumber ja välju.\n\
  -help\t\t\t\tnäita seda abiteksti ja välju.\n\n
$ #CompileOptions
# Kompileerimisoptsioonid:\n\
  Debuggimine\t\t\t%s\n\
  Kuju:\t\t\t%s\n\
  8bpp määratud teravustamine:\t%s\n\n
