$set 3 #Configmenu

$ #ConfigOptions
# Ayarlar
$ #FocusModel
# Foküsleme
$ #WindowPlacement
# Pencere yerleşimi
$ #ImageDithering
# Resim oluşturması
$ #OpaqueMove
# Ekranı içerikli taşı
$ #FullMax
# Tam ekranla, vallahi
$ #FocusNew
# Yeni pencereleri foküsle
$ #FocusLast
# Masaüstündeki son pencereyi foküsle
$ #ClickToFocus
# Tıklayarak foküsle
$ #SloppyFocus
# Ağır foküsle
$ #AutoRaise
# Otomatikman yükselt
$ #SmartRows
# Akıllı yerleşim( Sıralar )
$ #SmartCols
# Akıllı yerleşim( Sütunlar )
$ #Cascade
# Cascade Placement
$ #LeftRight
# Soldan sağa
$ #RightLeft
# Sağdan sola
$ #TopBottom
# Üstten aşağa
$ #BottomTop
# Alttan üste
$ #NoDithering
# Do not dither images
$ #OrderedDithering
# Use fast dither
$ #FloydSteinbergDithering
# Use high-quality dither
