$set 15 #Common

$ #Yes
# Jah
$ #No
# Ei

$ #DirectionTitle
# Suund
$ #DirectionHoriz
# Horisontaalne
$ #DirectionVert
# Vertikaalne

$ #AlwaysOnTop
# Alati esiplaanil

$ #PlacementTitle
# Asetus
$ #PlacementTopLeft
# Üleval vasakul
$ #PlacementCenterLeft
# Keskel vasakul
$ #PlacementBottomLeft
# All vasakul
$ #PlacementTopCenter
# Üleval keskel
$ #PlacementBottomCenter
# All keskel
$ #PlacementTopRight
# Üleval paremal
$ #PlacementCenterRight
# Keskel paremal
$ #PlacementBottomRight
# All paremal
