$set 5 #Image

$ #ErrorCreatingSolidPixmap
# BImage::render_solid : resmi yaratamadım\n
$ #ErrorCreatingXImage
# BImage::renderXImage : XImage'i yaratamadım\n
$ #UnsupVisual
# BImage::renderXImage : desteklenmeyen görünüş( renk derinliği )\n
$ #ErrorCreatingPixmap
# BImage::renderPixmap : resmi yaratamadım\n
$ #InvalidColormapSize
# BImageControl::BImageControl : geçersiz renk haritası büyüklüğü %d (%d/%d/%d) - azaltıyorum\n
$ #ErrorAllocatingColormap
# BImageControl::BImageControl : renk haritası ayrılanamadı\n
$ #ColorAllocFail
# BImageControl::BImageControl : rengi ayrırken hata oldu : %d/%d/%d\n
$ #PixmapRelease
# BImageControl::~BImageControl : resim arabelleği - %d resim temizlendi\n
$ #PixmapCacheLarge
# BImageControl::renderImage : arabellek büyük, temizlemeye başlıyorum\n
$ #ColorParseError
# BImageControl::getColor : renk tarama hatası : '%s'\n
$ #ColorAllocError
# BImageControl::getColor : renk ayırma hatası : '%s'\n
