$set 6 #Screen

$ #AnotherWMRunning
# BScreen::BScreen : X sunucusunu sorgularken hata oldu.\n  \
%s ekranında başka bir pencere yöneticisi çalışıyor gibi.\n
$ #ManagingScreen
# BScreen::BScreen : %d ekranı, 0x%lx görünümüyle , %d derinliğiyle\n
$ #FontLoadFail
# BScreen::LoadStyle() : '%s' yazı tipi yüklenemedi.\n
$ #DefaultFontLoadFail
# BScreen::LoadStyle(): önayarlı yazı tipi yüklenemedi.\n
$ #EmptyMenuFile
# %s : boş mönü dosyası\n
$ #xterm
# X komutası
$ #Restart
# Yeniden başla
$ #Exit
# Çık
$ #EXECError
# BScreen::parseMenuFile : [exec] hatası, mönü yaftası ve/yada komuta belirlenmedi\n
$ #EXITError
# BScreen::parseMenuFile : [exit] hatası, mönü yaftası belirlenmedi\n
$ #STYLEError
# BScreen::parseMenuFile : [style] hatası, mönü yaftası ve/yada dosya adı belirlenmedi\n
$ #CONFIGError
# BScreen::parseMenuFile: [config] hatası, mönü yaftası belirlenmedi\n
$ #INCLUDEError
# BScreen::parseMenuFile: [include] hatası, dosya adı belirlenmedi\n
$ #INCLUDEErrorReg
# BScreen::parseMenuFile: [include] hatası, '%s' vasat bir dosya değil\n
$ #SUBMENUError
# BScreen::parseMenuFile: [submenu] hatası, mönü yaftası belirlenmedi\n
$ #RESTARTError
# BScreen::parseMenuFile: [restart] hatası, mönü yaftası belirlenmedi\n
$ #RECONFIGError
# BScreen::parseMenuFile: [reconfig] hatası, mönü yaftası belirlenmedi\n
$ #STYLESDIRError
# BScreen::parseMenuFile: [stylesdir/stylesmenu] hatası, dizin adı belirlenmedi\n
$ #STYLESDIRErrorNotDir
# BScreen::parseMenuFile: [stylesdir/stylesmenu] hatası, '%s' bir dizin \
değildir\n
$ #STYLESDIRErrorNoExist
# BScreen::parseMenuFile: [stylesdir/stylesmenu] hatası, '%s' var değil\n
$ #WORKSPACESError
# BScreen::parseMenuFile: [workspaces] hatası, mönü yaftası belirlenmedi\n
$ #PositionLength
# 0: 0000 x 0: 0000
$ #PositionFormat
# X: %4d x Y: %4d
$ #GeometryFormat
# Y: %4d x E: %4d
