$set 16 #bsetroot

$ #MustSpecify
# %s: viga: pead määratlema ühe järgnevaist: -solid, -mod, -gradient\n
$ #Usage
# %s 2.0: (c) 1997-2000 Brad Hughes\n\n\
	  (c) 2001-2002 Sean 'Shaleh' Perry\n\n\
  -display <string>        displei ühendus\n\
  -mod <x> <y>             moodula tapeet\n\
  -foreground, -fg <color> esiplaani värvi moodula\n\
  -background, -bg <color> tausta värvi moodula\n\n\
  -gradient <texture>      gradientne tekstuur\n\
  -from <color>            gradiendi algusvärv\n\
  -to <color>              gradiendi lõpuvärv\n\n\
  -solid <color>           ühtne värv\n\n\
  -help                    näida seda abiteksti ja välju\n

