$set 5 #Image

$ #ErrorCreatingSolidPixmap
# BImage::render_solid: viga pixmapi loomisel\n
$ #ErrorCreatingXImage
# BImage::renderXImage: viga XImage loomisel\n
$ #UnsupVisual
# BImage::renderXImage: toeta visuaal\n
$ #ErrorCreatingPixmap
# BImage::renderPixmap: viga pixmapi loomisel\n
$ #InvalidColormapSize
# BImageControl::BImageControl: vale värvipaletti suurus %d (%d/%d/%d) - vähendame\n
$ #ErrorAllocatingColormap
# BImageControl::BImageControl: viga värvipaletti määramisel \n
$ #ColorAllocFail
# BImageControl::BImageControl: ei õnnestunud värvi määrata %d/%d/%d\n
$ #PixmapRelease
# BImageControl::~BImageControl: pixmap cache - vabastame %d pixmappi\n
$ #PixmapCacheLarge
# BImageControl::renderImage: cache on liiga suur, sundpuhastus\n
$ #ColorParseError
# BImageControl::getColor: viga värvi analüüsimisel: '%s'\n
$ #ColorAllocError
# BImageControl::getColor: viga värvi määramisel: '%s'\n
